@@ -7,8 +7,6 @@
 #'   \code{NULL} to skip launching a browser.
 #' @param dir The directory from which to to read input documents. Defaults to
 #'   the parent directory of \code{file}.
-#' @param default_file The file to serve at the Shiny server's root URL. If
-#'   \code{NULL} (the default), a sensible default is chosen (see Details)
 #' @param auto_reload If \code{TRUE} (the default), automatically reload the
 #'   Shiny application when the file currently being viewed is changed on disk.
 #' @param shiny_args Additional arguments to \code{\link[shiny:runApp]{runApp}}.
@@ -34,15 +32,6 @@
 #'   directory using standard Markdown syntax, e.g.
 #'   \code{[Analysis Page 2](page2.Rmd)}.
 #'
-#'   If \code{default_file} is not specified, nor is a file specified on the
-#'   URL, then the default document to serve at \code{/} is chosen from (in
-#'   order of preference):
-#'   \itemize{
-#'     \item{If \code{dir} contains only one \code{Rmd}, that \code{Rmd}.}
-#'     \item{The file \code{index.Rmd}, if it exists in \code{dir}}
-#'     \item{The file \code{index.html}, if it exists in \code{dir}}
-#'   }
-#'
 #'   If you wish to share R code between your documents, place it in a file
 #'   named \code{global.R} in \code{dir}; it will be sourced into the global
 #'   environment.
@@ -55,7 +44,7 @@
 #'   When using an external web browser with the server, specify the name of the
 #'   R Markdown file to view in the URL (e.g.
 #'   \code{http://127.0.0.1:1234/foo.Rmd}). A URL without a filename will show
-#'   the \code{default_file} as described above.
+#'   \code{index.Rmd}, if it exists in \code{dir}.
 #'
 #' @examples
 #' \dontrun{
@@ -68,52 +57,32 @@
 #'
 #' }
 #' @export
-run <- function(file = "index.Rmd", dir = dirname(file), default_file = NULL,
-                auto_reload = TRUE, shiny_args = NULL, render_args = NULL) {
+run <- function(file = "index.Rmd", dir = dirname(file), auto_reload = TRUE,
+                shiny_args = NULL, render_args = NULL) {
 
-  # select the document to serve at the root URL if not user-specified
-  if (is.null(default_file)) {
+  # if there's just one file in the directory, serve it
+  if (identical(file, "index.Rmd")) {
     allRmds <- list.files(path = dir, pattern = "^.*\\.[Rr][Mm][Dd]$")
     if (length(allRmds) == 1) {
-      # just one R Markdown document
-      default_file <- allRmds
-    } else {
-      # more than one: look for an index
-      index <- which(tolower(allRmds) == "index.rmd")
-      if (length(index) > 0) {
-        default_file <- allRmds[index[1]]
-      }
+      file <- allRmds
     }
   }
-
-  if (is.null(default_file)) {
-    # no R Markdown default found; how about an HTML?
-    indexHtml <- list.files(path = dir, pattern = "index.html?",
-                            ignore.case = TRUE)
-    if (length(indexHtml) > 0) {
-      default_file <- indexHtml[1]
-    }
-  }
-
+  
   # form and test locations
-  dir <- normalize_path(dir)
-  if (!dir_exists(dir))
+  dir <- normalizePath(dir, winslash="/")
+  if (!file.exists(dir))
     stop("The directory '", dir, "' does not exist")
 
   if (!is.null(file)) {
     # compute file path relative to directory (remove common directory prefix
     # if it exists)
-    file_rel <- normalize_path(file)
+    file_rel <- normalizePath(file, winslash = "/")
     if (identical(substr(file_rel, 1, nchar(dir)), dir))
       file_rel <- substr(file_rel, nchar(dir) + 2, nchar(file_rel))
 
-    # if we don't have a default to launch, make sure the user-specified file
-    # exists
-    if (is.null(default_file)) {
-      resolved <- resolve_relative(dir, file_rel)
-      if (is.null(resolved) || !file.exists(resolved))
-        stop("The file '", file, "' does not exist in the directory '", dir, "'")
-    }
+    resolved <- resolve_relative(dir, file_rel)
+    if (is.null(resolved) || !file.exists(resolved))
+      stop("The file '", file, "' does not exist in the directory '", dir, "'")
   }
 
   # pick up encoding
@@ -134,11 +103,11 @@ run <- function(file = "index.Rmd", dir = dirname(file), default_file = NULL,
   # combine the user-supplied list of Shiny arguments with our own and start
   # the Shiny server; handle requests for the root (/) and any R markdown files
   # within
-  app <- shiny::shinyApp(ui = rmarkdown_shiny_ui(dir, default_file),
-                         uiPattern = "^/$|^/index\\.html?$|^(/.*\\.[Rr][Mm][Dd])$",
+  app <- shiny::shinyApp(ui = rmarkdown_shiny_ui(dir, file),
+                         uiPattern = "^/$|^(/.*\\.[Rr][Mm][Dd])$",
                          onStart = onStart,
                          server = rmarkdown_shiny_server(
-                           dir, default_file, encoding, auto_reload, render_args))
+                           dir, file, encoding, auto_reload, render_args))
 
   # launch the app and open a browser to the requested page, if one was
   # specified
@@ -190,10 +159,7 @@ rmarkdown_shiny_server <- function(dir, file, encoding, auto_reload, render_args
 
       # if output is cached, return it directly
       if (out$cached) {
-        if (nchar(out$resource_folder) > 0) {
-          shiny::addResourcePath(basename(out$resource_folder),
-                                 out$resource_folder)
-        }
+        shiny::addResourcePath(basename(out$resource_folder), out$resource_folder)
         return (out$shiny_html)
       }
 
@@ -231,13 +197,12 @@ rmarkdown_shiny_server <- function(dir, file, encoding, auto_reload, render_args
                                output_dir = dirname(output_dest),
                                output_options = output_opts,
                                intermediates_dir = dirname(output_dest),
-                               runtime = "shiny",
-                               envir = new.env()),
+                               runtime = "shiny"),
                           render_args)
       result_path <- shiny::maskReactiveContext(do.call(render, args))
 
       # ensure the resource folder exists, and map requests to it in Shiny
-      if (!dir_exists(resource_folder))
+      if (!file.exists(resource_folder))
         dir.create(resource_folder, recursive = TRUE)
       shiny::addResourcePath(basename(resource_folder), resource_folder)
 
@@ -259,7 +224,10 @@ rmarkdown_shiny_server <- function(dir, file, encoding, auto_reload, render_args
           unlink(resource_folder, recursive = TRUE)
         })
       }
-      shinyHTML_with_deps(result_path, dependencies)
+      attachDependencies(
+        htmltools::HTML(paste(readLines(result_path, encoding = "UTF-8", warn = FALSE),
+                          collapse="\n")),
+        dependencies)
     })
     output$`__reactivedoc__` <- shiny::renderUI({
       doc()
@@ -277,11 +245,8 @@ rmarkdown_shiny_ui <- function(dir, file) {
       req_path <- file
     }
 
-    # request must be for an R Markdown or HTML document
-    ext <- tolower(tools::file_ext(req_path))
-    if (!identical(ext, "rmd") &&
-        !identical(ext, "htm") &&
-        !identical(ext, "html")) {
+    # request must be for an R Markdown document
+    if (!identical(tolower(tools::file_ext(req_path)), "rmd")) {
       return(NULL)
     }
 
@@ -312,11 +277,11 @@ rmarkdown_shiny_ui <- function(dir, file) {
   }
 }
 
-shinyHTML_with_deps <- function(html_file, deps) {
-  htmltools::attachDependencies(
-    htmltools::HTML(paste(readLines(html_file, encoding = "UTF-8", warn = FALSE),
-                          collapse = "\n")),
-    deps)
+shinyHTML_with_deps <- function (html_file, deps) {
+  structure(
+    shiny::HTML(paste(readLines(html_file, encoding = "UTF-8", warn = FALSE),
+                      collapse="\n")),
+    html_dependency = deps)
 }
 
 # given an input file and its encoding, return a list with values indicating
@@ -329,17 +294,6 @@ rmd_cached_output <- function (input, encoding) {
   shiny_html <- NULL
   resource_folder <- ""
 
-  # if the file is raw HTML, return it directly
-  if (identical(tolower(tools::file_ext(input)), "htm") ||
-      identical(tolower(tools::file_ext(input)), "html")) {
-    return(list(
-      cacheable = TRUE,
-      cached = TRUE,
-      dest = "",
-      shiny_html = shinyHTML_with_deps(input, NULL),
-      resource_folder = ""))
-  }
-
   # check to see if the file is a Shiny document
   front_matter <- parse_yaml_front_matter(read_lines_utf8(input, encoding))
   if (!identical(front_matter$runtime, "shiny")) {
@@ -384,8 +338,8 @@ resolve_relative <- function(dir, relpath) {
   abs.path <- file.path(dir, relpath)
   if (!file.exists(abs.path))
     return(NULL)
-  abs.path <- normalize_path(abs.path, mustWork = TRUE)
-  dir <- normalize_path(dir, mustWork = TRUE)
+  abs.path <- normalizePath(abs.path, winslash='/', mustWork=TRUE)
+  dir <- normalizePath(dir, winslash='/', mustWork=TRUE)
   # trim the possible trailing slash under Windows
   if (.Platform$OS.type == 'windows') dir <- sub('/$', '', dir)
   if (nchar(abs.path) <= nchar(dir) + 1)
@@ -402,7 +356,7 @@ file.path.ci <- function(dir, name) {
   default <- file.path(dir, name)
   if (file.exists(default))
     return(default)
-  if (!dir_exists(dir))
+  if (!file.exists(dir))
     return(default)
 
   matches <- list.files(dir, name, ignore.case=TRUE, full.names=TRUE,
